import java.io.File;
import java.util.ArrayList;
import java.util.Scanner;

// This class only allows simple graphs (no loops and no multiple edges)
public class SimpleGraph {
	
	/********************************************
	 * 											*
	 *           instance variables				*
	 * 											*
	 ********************************************/
	
	private ArrayList<Vertex> vertices;  // vertices of graph
    private ArrayList<Edge> edges;  // edges of graph
    
    /********************************************
	 * 											*
	 *          getters and setters				*
	 * 											*
	 ********************************************/
    
    public ArrayList<Vertex> getVertices() {
		return vertices;
	}
	
	public Vertex addVertex(Vertex vertex) {
		// Does not allow two vertices with one label
		if (vertexExists(vertex))
			return findVertex(vertex);
		else{
			this.vertices.add(vertex);
			return vertex;
		}
	}
	
	public ArrayList<Edge> getEdges() {
		return edges;
	}
	
	public void addEdge(Edge edge) {
    	// Does not allow self-loop
    	if (edge.getV1() != edge.getV2()){
    		// Does not allow multiple edges
    		if (!edgeExists(edge)){
    			edge.getV1().addAdjacentEdge(edge);
    			edge.getV2().addAdjacentEdge(edge);
    	        this.edges.add(edge);
    		}
    	}
	}
	
	public int n() {
		return vertices.size();
	}
	
	public int m() {
		return edges.size();
	}
	
	/********************************************
	 * 											*
	 *              Constructors				*
	 * 											*
	 ********************************************/
	
	// First constructor
    public SimpleGraph(String fileName){
    	this.vertices = new ArrayList<Vertex>();
    	this.edges = new ArrayList<Edge>();
    	this.readGraphfromFile(fileName);
    }
    
    /********************************************
	 * 											*
	 *              Search methods				*
	 * 											*
	 ********************************************/
    
    // Finds a vertex by its label
    public Vertex findVertex(Vertex vertex){
    	return findVertexByLabel(vertex.getLabel());
    }
    
    // Finds a vertex by its label
    public Vertex findVertexByLabel(String label){
    	for (int i=0; i<vertices.size(); i++){
    		if (vertices.get(i).getLabel().equals(label))
    			return vertices.get(i);
    	}
    	return null;
    }
    
    // Finds an edge by its two end points
    public Edge findEdge(Edge edge){
    	return findEdgeByTwoVertices(edge.getV1(), edge.getV2());
    }
    
    // Finds an edge by its two end points
    public Edge findEdgeByTwoVertices(Vertex v1, Vertex v2){
    	for (int i=0; i<edges.size(); i++){
    		if ((edges.get(i).getV1() == v1 && edges.get(i).getV2() == v2) || (edges.get(i).getV2() == v1 && edges.get(i).getV1() == v2))
    			return edges.get(i);
    	}
    	return null;
    }
    
    /********************************************
	 * 											*
	 *              Other methods				*
	 * 											*
	 ********************************************/    
    
    // Returns true if a vertex exists or false if it does not exist
    public boolean vertexExists(Vertex vertex){
    	if (findVertex(vertex) != null)
    		return true;
    	else
    		return false;
    }
    
    // Returns true if an edge exists or false if it does not exist
    public boolean edgeExists(Edge edge){
    	if (findEdge(edge) != null)
    		return true;
    	else
    		return false;
    }
    
    /********************************************
	 * 											*
	 *          Input/output methods			*
	 * 											*
	 ********************************************/
    
    // Reads the graph from file 
    public void readGraphfromFile(String fileName){
    	Vertex v1;
    	Vertex v2;
    	try{
			File file = new File(fileName);
			Scanner fileScanner = new Scanner(file);
			
			//ignores the first two integers in the input file
			if (fileScanner.hasNext())
				fileScanner.nextInt();
			if (fileScanner.hasNext())
				fileScanner.nextInt();
			
			while (fileScanner.hasNext()) {
				String l1 = fileScanner.nextInt()+"";  // Label of the first vertex in edge
				String l2 = fileScanner.nextInt()+"";  // Label of the second vertex in edge
				int weight = fileScanner.nextInt();  // Weight of the edge
				
				v1 = new Vertex(l1);
				v1 = addVertex(v1);
				
				v2 = new Vertex(l2);
				v2 = addVertex(v2);
				
				addEdge(new Edge(v1,v2,weight));
			}
		}
		catch (Exception e) {
			System.out.printf("Error in the graph input file: "+e.getMessage()+"\n");
		}
	}
    
    // Produces the adjacency matrix of the graph
    public int[][] adjacencyMatrix(){
		int[][] adjacencyMatrix = new int[this.vertices.size()][this.vertices.size()];
    	for (int i=0; i<vertices.size(); i++){
			for (int j=0; j<vertices.size(); j++) {
				Edge edge = new Edge(vertices.get(i), vertices.get(j));
				if (edgeExists(edge))
					adjacencyMatrix[i][j] = findEdge(edge).getWeight();
				else
					adjacencyMatrix[i][j] = 0;
			}
		}
    	return adjacencyMatrix;
	}
    
    // Prints the adjacency matrix of the graph with weights of the edges. If it indicates 0 it means that there is no edge between the two vertices.
    public void printGraph(){
		int[][] adjacencyMatrix = adjacencyMatrix();
		
		System.out.println("");
		for (int i=0; i<vertices.size(); i++){
			for (int j=0; j<vertices.size(); j++) {
				System.out.printf("%-6d",adjacencyMatrix[i][j]);
			}
			System.out.println("");
		}
	}
    
    /********************************************
	 * 											*
	 *               Main method				*
	 * 											*
	 ********************************************/
    
//	public static void main (String[] args) { 
//		SimpleGraph G = new SimpleGraph(args[0]);
//		G.printGraph();
//	}

}
